/****************************************************************************

    display.c
    displays game board

    DoubleDash game
      
    Copyright (C) 2003  Michal Benes
  
    DoubleDash is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.
  
    This code is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.
  
    You should have received a copy of the GNU Lesser General Public
    License along with this code; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
                                                                               
*****************************************************************************/

#include "doubledash.h"
#include "display.h"
#include "game.h"
#include "player.h"
#include "sdl.h"
#include "text.h"


Sdl_Sprite *title_free;
Sdl_Sprite *title_dirt;
Sdl_Sprite *title_barrier;
Sdl_Sprite *title_boulder;
Sdl_Sprite *title_diamond;
Sdl_Sprite *title_player;
Sdl_Sprite *title_myplayer;
Sdl_Sprite *title_wall;
Sdl_Sprite *title_firefly;
Sdl_Sprite *title_butterfly;
Sdl_Sprite *title_explosion;
Sdl_Sprite *title_door;
Sdl_Sprite *title_curtain;


#define CURTAIN_DURATION 23

/*
  load titles
*/
void display_load_titles(void)
{
  title_free=sdl_sprite_load( "empty.png", 4, 1 );
  title_dirt=sdl_sprite_load( "dirt.png", 1, 1 );
  title_barrier=sdl_sprite_load( "barrier.png", 1, 1 );
  title_boulder=sdl_sprite_load( "boulder.png", 1, 1 );
  title_diamond=sdl_sprite_load( "diamond.png", 8, 1 );
  title_player=sdl_sprite_load( "gnome.png", 8, 5 );
  title_myplayer=sdl_sprite_load( "mygnome.png", 8, 5 );
  title_wall=sdl_sprite_load( "wall.png", 4, 1 );
  title_firefly=sdl_sprite_load( "firefly.png", 8, 1 );
  title_butterfly=sdl_sprite_load( "butterfly.png", 8, 1 );
  title_explosion=sdl_sprite_load( "explosion.png", 3, 1 );
  title_door=sdl_sprite_load( "door.png", 2, 1 );
  title_curtain=sdl_sprite_load( "curtain.png", 4, 1 );
}



/*
  displays single player board
 */
void display_player( int id, int frame, int xoffset, int yoffset, int width, int height)
{
  int titlew, titleh;
  int mapw, maph;
  int viewx, viewy;
  int startx, starty;
  int x, y;
  int a;
  int i, j;
  int plid;
  int title;
  Sdl_Sprite *sprite;
  int sprite_frame=0;
  char buf[20];

  sdl_clip( xoffset, yoffset, width, height );

  titlew=title_free->image[0]->w;
  titleh=title_free->image[0]->h;

  game_get_map_dimen( &mapw, &maph );
  player_get_view( id, &viewx, &viewy, titlew, titleh );

  if( viewx+width/2>mapw*titlew ) viewx=mapw*titlew-width/2;
  if( viewy+height/2>maph*titleh ) viewy=maph*titleh-height/2;
  
  if( viewx-width/2<0 ) viewx=width/2;
  if( viewy-height/2<0 ) viewy=height/2;

  startx=(viewx-width/2)/titlew;
  starty=(viewy-height/2)/titleh;

  for( j=0; j<height/titleh+2; ++j )
    for( i=0; i<width/titlew+2; ++i )
      {
	x=startx+i;
	y=starty+j;

	title=game_get_map_title( x, y );
	
	if( frame/2<CURTAIN_DURATION )
	  {
	    a=2*x*(x+y)+3*y*217;
	    if( ((a*a)%CURTAIN_DURATION)
	            *CURTAIN_DURATION*CURTAIN_DURATION
		<=(CURTAIN_DURATION-frame/2)*(CURTAIN_DURATION-frame/2)*(CURTAIN_DURATION-frame/2) )	      
	      title=-1;
	  }

	switch( title )
	  {
	  case -1: /* curtain */
	    sprite=title_curtain;
	    sprite_frame=frame%4;
	    break;

	  case MAP_BARRIER:
	    sprite=title_barrier;
	    sprite_frame=0;
	    break;
	  case MAP_FREE:
	    sprite=title_free;
	    sprite_frame=player_get_flash(id);
	    break;
	  case MAP_DIRT:
	    sprite=title_dirt;
	    sprite_frame=0;
	    break;
	  case MAP_BOULDER:
	  case MAP_FALLING_BOULDER:
	    sprite=title_boulder;
	    sprite_frame=0;
	    break;
	  case MAP_DIAMOND:
	  case MAP_FALLING_DIAMOND:
	    sprite=title_diamond;
	    sprite_frame=frame%8;
	    break;
	  case MAP_PLAYER:
	    plid=player_get_id(x,y);
	    if( id==plid )
	      sprite=title_myplayer;
	    else
	      sprite=title_player;
	    sprite_frame=player_frame(plid,frame);
	    break;
	  case MAP_EGG:
	    if( frame+2>=BORN_TIME )
	      {
		sprite=title_explosion;
		sprite_frame=frame%2;
		break;
	      }
	    plid=player_get_id(x,y);
	    sprite=title_door;
	    sprite_frame=0;
	    if( id==plid && frame/2%2 )
	      sprite_frame=1;
	    break;
	  case MAP_WALL:
	    sprite=title_wall;
	    sprite_frame=0;
	    break;
	  case MAP_FIREFLY:
	    sprite=title_firefly;
	    sprite_frame=frame%8;
	    break;
	  case MAP_BUTTERFLY:
	    sprite=title_butterfly;
	    sprite_frame=frame%8;
	    break;
	  case MAP_DIAMOND_EXPLOSION:
	  case MAP_FREE_EXPLOSION:
	    sprite=title_explosion;
	    sprite_frame=frame%2+1;
	    break;
	  case MAP_EXIT:
	    sprite=title_door;
	    sprite_frame=0;
	    if( (frame/2)%2==1 && player_get_success( id ) ) 
	      sprite_frame=1;
	    break;

	  default:
	    sprite=NULL;
	  }

	if( sprite!=NULL )
	  sdl_sprite_draw( sprite, 
			   xoffset+width/2+x*titlew-viewx, 
			   yoffset+height/2+y*titleh-viewy, sprite_frame ); 
	  

      }

  
  sprintf( buf, "%d/%d", player_get_score(id), game_get_level_diamonds() );
  text_write( buf, xoffset+5, yoffset+5, id, 1 );

  sprintf( buf, "%d", player_get_total_score(id) );
  text_write( buf, xoffset+width-5, yoffset+5, id, -1 );

  sdl_unclip();
}



/*
  display game board
 */
void display_update( int players_num, int frame )
{
  int w, h;
  if( title_free==NULL )
    display_load_titles();

  if( players_num==1 )
    {
      display_player( 0, frame, 1, 1, SCREEN_WIDTH-2, SCREEN_HEIGHT-2 );
    }

  if( players_num==2 )
    {
      w=SCREEN_WIDTH/2;
      h=SCREEN_HEIGHT;
      display_player( 0, frame, 1, 1, w-2, h-2 );
      display_player( 1, frame, w+1, 1, w-2, h-2 );
    }
     
  if( players_num==3 )
    {
      w=SCREEN_WIDTH/2;
      h=SCREEN_HEIGHT/2;
      display_player( 0, frame, 1, 1, w-2, h-2 );
      display_player( 1, frame, w+1, 1, w-2, h-2 );
      display_player( 2, frame, w-w/2+1, h+1, w-2, h-2 );
    }
     
  if( players_num==4 )
    {
      w=SCREEN_WIDTH/2;
      h=SCREEN_HEIGHT/2;
      display_player( 0, frame, 1, 1, w-2, h-2 );
      display_player( 1, frame, w+1, 1, w-2, h-2 );
      display_player( 2, frame, 1, h+1, w-2, h-2 );
      display_player( 3, frame, w+1, h+1, w-2, h-2 );
    }

  sdl_flip();
     
}



