/****************************************************************************

    player.h
    player handling

    DoubleDash game
      
    Copyright (C) 2003  Michal Benes
  
    DoubleDash is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.
  
    This code is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.
  
    You should have received a copy of the GNU Lesser General Public
    License along with this code; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
                                                                               
*****************************************************************************/


#ifndef __PLAYER_H
#define __PLAYER_H


/*
  init player in a new level
 */
void player_init( int id, int x, int y, int view_x, int view_y );


/*
  null player score
 */
void player_null_score();


/*
  get players view center
 */
void player_get_view( int id, int *viewx, int *viewy, int titlew, int titleh );


/*
  Do player next frame
 */
void player_next_frame( int id, int major );


/*
  Get player score
 */
int player_get_score( int id );


/*
  Get player total score
 */
int player_get_total_score( int id );

/*
  Add player total score
 */
void player_add_total_score( int id, int value );


/*
  set player keys
 */
void player_set_keys( int id, int up, int right, int down, int left, int dig );


/*
  check keypress
 */
void player_keypress( int key );



/*
  check keydepress
 */
void player_keydepress( int key );



/*
  Get player movement direction
*/
void player_get_dir( int id, int *dx, int *dy, int *dig );


/*
  Let player move
*/
void player_move( int id );


/*
  Player got diamond
*/
void player_got_diamond( int id );


/*
  Player can move flag
*/
void player_can_move( int id, int value );


/*
  Get player id
*/
int player_get_id( int x, int y );


/*
  Has player succeded?
*/
int player_get_success( int id );


/*
  Player succeded
*/
void player_set_success( int id );


/*
  Player get flash state
*/
int player_get_flash( int id );


/*
  Get player frame
*/
int player_frame( int id, int frame );


/*
  Player is dead
 */
void player_dead( int id );


#endif /* __PLAYER_H */
